<?php

class iw_assets {

	protected $groups = array();
	protected $admin = false;
	protected $debug = false;
	protected $config = '/files/addons/iw_assets/config.txt';


	public function __construct () {
		global $REX;
		if(isset($REX['LOGIN']) && is_object($REX['LOGIN']) && is_object($REX['LOGIN']->USER) && $REX['LOGIN']->USER->isAdmin())
		{
			$this->admin = true;
		}
	}


	/*
	 * activates debug mode (files will not be minified)
	 */
	public function debug ()
	{
		$this->debug = true;
	}


	/*
	 * adds assets
	 *
	 * @param $filepath: complete path to file STRING
	 * @param $groupname optional STRING
	 */
	public function add ($filepath, $groupname = 'default')
	{
		$this->check_filepath($filepath);
		$ext = substr($filepath, (strrpos($filepath, '.') + 1));
		if ($ext === 'less')
		{
			if($this->admin)
			{
				$absfilepath = str_replace('//', '/', $_SERVER['DOCUMENT_ROOT'].'/'.$filepath);
				$this->compile_less($absfilepath, str_replace('.less', '.css', $absfilepath));
			}
			$filepath = str_replace('.less', '.css', $filepath);
			$ext = 'css';
		}
		$this->groups[$groupname][$ext][] = $filepath;
		file_put_contents($_SERVER['DOCUMENT_ROOT'].$this->config, serialize($this->groups));
	}


	/*
	 * returns <link> reference(s) for a group
	 *
	 * @param $groupname STRING
	 * @param $mediatype: eg "all"/"screen"/"print" STRING
	 */
	public function get_css ($groupname = 'default', $mediatype = 'all')
	{
		$debug = '';
		$filepaths = array();
		if (isset($this->groups[$groupname])
			&& isset($this->groups[$groupname]['css'])
			&& count($this->groups[$groupname]['css']))
		{
			foreach ($this->groups[$groupname]['css'] as $filepath)
			{
				$debug .= '<link rel="stylesheet" href="'.$filepath.'" type="text/css" media="'.$mediatype.'" />';
				$filepaths[] = $filepath;
			}
			return ($this->debug) ? $debug : '<link rel="stylesheet" href="/files/addons/iw_assets/index.php?task=get_css&amp;group='.$groupname.'" type="text/css" media="'.$mediatype.'" />';
		}
	}


	/*
	 * returns <script> reference for a group
	 *
	 * @param $groupname STRING
	 */
	public function get_js ($groupname = 'default')
	{
		$debug = '';
		$filepaths = array();
		if (isset($this->groups[$groupname])
			&& isset($this->groups[$groupname]['js'])
			&& count($this->groups[$groupname]['js']))
		{
			foreach ($this->groups[$groupname]['js'] as $filepath)
			{
				$debug .= '<script src="'.$filepath.'"></script>';
				$filepaths[] = $filepath;
			}
			return ($this->debug) ? $debug : '<script src="/files/addons/iw_assets/index.php?task=get_js&amp;group='.$groupname.'"></script>';
		}
	}


	/*
	 * checks if filepath is valid
	 *
	 * @param $filepath STRING
	 */
	protected function check_filepath ($filepath)
	{
		$filepath = str_replace('//', '/', $_SERVER['DOCUMENT_ROOT'].'/'.$filepath);
		if ($this->admin && !is_file($filepath))
		{
			die('iw_assets: '.$filepath.' ist nicht vorhanden');
		}
	}



	/*
	 * cachedCompile() autocompiles .less files form @import. to recognize changes
	 * in files from @import a cache file is needed
	 *
	 * @param $inputFile STRING
	 * @param $outputFile STRING
	 */
	protected function compile_less ($inputFile, $outputFile) {
		$cacheFile = $inputFile.".cache";
		if (file_exists($cacheFile)) {
			$cache = unserialize(file_get_contents($cacheFile));
		} else {
			$cache = $inputFile;
		}
		$less = new lessc;
		$newCache = $less->cachedCompile($cache);
		if (!is_array($cache) || $newCache['updated'] > $cache['updated']) {
			file_put_contents($cacheFile, serialize($newCache));
			file_put_contents($outputFile, $newCache['compiled']);
		}
	}

}
?>
